{
  StepperDriveErz
  Copyright 2012. Bruce Drummond.
  Release Date: 02/28/2012

  You may use and modify this software without any restriction.

  This is the most complete, concise, and comprehensive release of my
  work pertaining to stepper drivers.  If you find any errors with
  these drivers, please let me know.  My Parallax forum user name
  is idbruce.

  I do not intend to discuss all the revisions, so you will have to
  accept it on an as basis.  

  The main settings for affecting motor operation are:
  
    START_STOP_SPEED, RAMP_INC_DEC, and MAX_EXE_MOTOR_SPEED 

  By altering these three settings, a person can achieve many
  different ramping and operating profiles.  So go ahead and experiment
  a little and have some fun.      
}

CON

  'Multiply the external frequency by 16.
  _CLKMODE              = XTAL1 + PLL16X

  'Specify the speed of the external crystal.
  _XINFREQ              = 5_000_000

  'Please note that the following settings are for a GeckoDrive
  'G251X microstepping driver in combination with an Applied
  'Motion HT23-400 high torque stepper motor.  You may have to
  'alter these settings to suit your particular driver, motor,
  'and needs.

  'Establish the clock frequency so that a couple constants may
  'be declared in the CON block.
  CLK_FREQ              = ((_CLKMODE - XTAL1) >> 6) * _XINFREQ

  'The GeckoDrive G251X microstepping driver requires a minimal
  'pulse width of 1us, so establish this setting.
  HIGH_PULSE_WIDTH      = CLK_FREQ / 1_000_000

  'The code needs a certain amount of time to operate, so set
  'this.  Additonally, this setting can be used to control the
  'maximum speed of the motor.  Just remember that the maximum
  'execution speed takes priority over maximum speed of the
  'motor, otherwise the code will falter.  The current setting is
  'very close to maximum speed for SPIN.
  MAX_EXE_MOTOR_SPEED   = CLK_FREQ / 20_000  'Very close to maximum 

  'This setting is the starting and stopping speed of the
  'motor.  Please note that the RAMP will be subtracted from
  'this setting during the first loop.  Larger divisors of
  'CLK_FREQ will increase the initial startup speed of the motor,
  'whereas smaller divisors will have the opposite affect, thus
  'decreasing the initial startup speed of the motor.  The current
  'setting is very close to maximum setting for the combination of
  'a G251X stepper driver and HT23-400 motor.  If an attempt is made
  'to start the motor at too high a speed, the motor will not overcome
  'the moment of inertia and the motor will either hum, shutter, or shake :)
  START_STOP_SPEED      = CLK_FREQ / 1_600  'Very close to maximum

  'This setting is the ramp incrementor/decrementor, and it will
  'affect just how fast the motor ramps up to top speed and
  'ramps down until the motor stops.  Larger divisors of
  'CLK_FREQ will decrease the ramp up and ramp down speed of the
  'motor, whereas smaller divisors will have the opposite affect,
  'thus increasing the ramp up and ramp down speed of the
  'motor.  The current setting is very close to maximum setting for
  'the combination of a G251X stepper driver and HT23-400 motor.
  'Basically this is a setting of 1 - 25.
  RAMP_INC_DEC          = CLK_FREQ / 3_200_000 'Very close to maximum

  'Set the pin number for an input to ramp the motor down to a complete stop.
  STOP_INPUT_PIN        = 7

  'Set the number of steps per revolution.  This is necessary to determine the
  'number of steps to complete in order to stop on a completed revolution when
  'using the StopOnFullRev function.  This setting assumes a motor with 1.8
  'degree step angle used in conjunction with stepper driver having a 1/10
  'microstepping resolution.
  STEPS_PER_REV         = 2_000  

  'Set the pin numbers for the step, direction, and disable pins
  'for the stepper driver.
  DIRECTION_PIN         = 3
  STEP_PIN              = 4  
  DISABLE_PIN           = 5

PUB Main

  'Set the direction of the STOP_INPUT_PIN as an input.
  DIRA[STOP_INPUT_PIN]~

  'Set the directions of the direction, step, and disable pins
  'for the stepper driver as outputs.
  DIRA[STEP_PIN]~~
  DIRA[DIRECTION_PIN]~~
  DIRA[DISABLE_PIN]~~

  'Enable the motor.
  OUTA[DISABLE_PIN]~~

  'Set the direction of rotation.
  OUTA[DIRECTION_PIN]~

  'Please note that a pushbutton is being utilized for STOP_INPUT_PIN.
  'As shown in the loop below, this button is used to activate the
  'stepper drivers, however this is just for testing purposes.  The
  'real purpose of the STOP_INPUT_PIN is to inform the RunDaStepperMotor
  'function that the stepper motor should come to a stop.  In a real
  'world setting, a detent switch, an optical switch, a proximity sensor,
  'or other type of input may be used to stop the motor and to permit
  'a more automated process.

  'Loop until there is input on the PUSHBUTTON_PIN and then stop the iteration.
  REPEAT
  
    IF INA[STOP_INPUT_PIN]

      QUIT

  'Please note that in the following calls, most of the function parameters are
  'passed constants.  It is not necessary to pass constants to these functions,
  'but in certain instances it is highly desirable.  For instance, each stepper motor
  'will probably have a different load put upon it, and many times a given stepper
  'motor will be required to move the same reoccuring load. In this type of instance,
  'it would be highly desirable to provide as many constant settings as possible,
  'and if possible, each motor should have it's own set of operating constants.
  'In the following function calls, documentation is provided under the assumption
  'that the provided constants have not been changed.  However, it may be necessary to
  'alter these constants to suit your particular motor(s) and/or stepper driver(s). 

  'In this example a call is made to the StopOnFullRev function, which stops the motor on a completed
  'revolution.  This function can be useful for cam operated drive applications.  However,
  'in order to compensate for the extra necessary computations to achieve a completed revolution,
  'the maximum execution speed of this function must be reduced, so for this example we will hard
  'code the reduced maximum speed into the function call.  For further information, please refer
  'to the function documentation.   
  StopOnFullRev(STEP_PIN, STOP_INPUT_PIN, START_STOP_SPEED, CLK_FREQ / 15_000, RAMP_INC_DEC, STEPS_PER_REV)

  'Here a delay is being applied to allow for inspection.
  WAITCNT(CLKFREQ * 10 + CNT)   

  'In this example a call is made to the RunDaStepperMotor.  For further information,
  'please refer to the function documentation.       
  RunDaStepperMotor(STEP_PIN, STOP_INPUT_PIN, START_STOP_SPEED, MAX_EXE_MOTOR_SPEED, RAMP_INC_DEC)

  'Here a delay is being applied to allow for motor position inspection.
  WAITCNT(CLKFREQ * 10 + CNT)

  'In this example a call is made to the PulseTheStepPin function with enough TotalSteps
  'specified to ramp up to maximum speed, maintain maximum speed for a short while, and then
  'ramp down until coming to a complete stop.  Using a G251X stepper driver, with a
  '1/10 microstep resolution, this call will have the motor make 100 complete revolutions
  'before stopping.  For further information, please refer to function documentation.       
  PulseTheStepPin(200_000, STEP_PIN, START_STOP_SPEED, MAX_EXE_MOTOR_SPEED, RAMP_INC_DEC)

  'Here a delay is being applied to allow for motor position inspection.
  WAITCNT(CLKFREQ * 10 + CNT)

  'In this example a call is made to the PulseTheStepPin function with an insufficient amount
  'of TotalSteps for the motor to ramp up to maximum speed.  In this situation, if TotalSteps
  'is divisible by 2, the motor will ramp up to 50% of the total steps and then begin a ramp
  'down until coming to a complete stop.  If TotalSteps is not divisible by 2, the motor will
  'ramp up to the dividend, and then move an additional step for the remainder, and then begin
  'a ramp down until coming to a complete stop.  For further information, please refer to
  'function documentation.       
  PulseTheStepPin(1_500, STEP_PIN, START_STOP_SPEED, MAX_EXE_MOTOR_SPEED, RAMP_INC_DEC) 

PUB RunDaStepperMotor(StepPin, StopInputPin, CycleOffset, MaxExeMotorSpeed, RampIncDec) | RampingSteps, Counter

  'This function will ramp up a stepper motor until the maximum speed
  'is obtained and it will maintain that speed until STOP_INPUT_PIN goes
  'high, at which point it will begin a ramp down of the stepper motor
  'until it comes to a stop.  Please note that the minimum steps required for
  'this function is RampingSteps * 2.

  'Determine the amount of ramping steps necessary before maximum speed
  'is obtained.
  RampingSteps := (CycleOffset - MaxExeMotorSpeed) / RampIncDec  

  'Set up the CTRMODE of Counter A for NCO/PWM single-ended.
  CTRA[30..26] := %00100

  'Set the output pin for Counter A.
  CTRA[5..0] := StepPin

  'Set the value to be added to PHSA with every clock cycle.
  FRQA := 1

  'Set APIN as an output.
  DIRA[StepPin]~~

  'Get the current System Counter value.
  Counter := CNT

  'Ramp up motor speed 
  REPEAT RampingSteps

    'Send out a high pulse on the step pin for the desired duration.
    PHSA := -HIGH_PULSE_WIDTH

    'Wait for a specified period of time before send sending another
    'high pulse to the step pin.
    WAITCNT(Counter += CycleOffset -= RampIncDec)

  'Run the motor at full speed until we get contrary input and then
  'stop this iteration. 
  REPEAT WHILE NOT INA[StopInputPin]
 
    'Send out a high pulse on the step pin for the desired duration.
    PHSA := -HIGH_PULSE_WIDTH

    'Wait for a specified period of time before send sending another
    'high pulse to the step pin.
    WAITCNT(Counter += CycleOffset)

  'Ramp down the motor and come to a stop.
  REPEAT RampingSteps
 
    'Send out a high pulse on the step pin for the desired duration.
    PHSA := -HIGH_PULSE_WIDTH

    'Wait for a specified period of time before send sending another
    'high pulse to the step pin.
    WAITCNT(Counter += CycleOffset += RampIncDec)

PUB StopOnFullRev(StepPin, StopInputPin, CycleOffset, MaxExeMotorSpeed, RampIncDec, StepsPerRev) | RampingSteps, TotalRampingSteps, RunningSteps, Counter, ExtraSteps

  'This function will ramp up a stepper motor until the maximum speed
  'is obtained and it will maintain that speed until STOP_INPUT_PIN goes
  'high, at which point it will move some necessary extra steps to enable
  'stopping on a completed revolution.  After executing the extra steps,
  'the motor will begin to ramp down until it comes to a stop.  Please note
  'that the minimum steps required for this function is RampingSteps * 2,
  'plus any additonal overhead necessary for input or stopping on a
  'completed revolution.

  'Determine the amount of ramping steps necessary before maximum speed
  'is obtained.
  RampingSteps := (CycleOffset - MaxExeMotorSpeed) / RampIncDec

  'Determine the total amount of ramping steps.
  TotalRampingSteps := RampingSteps * 2  

  'Set up the CTRMODE of Counter A for NCO/PWM single-ended.
  CTRA[30..26] := %00100

  'Set the output pin for Counter A.
  CTRA[5..0] := StepPin

  'Set the value to be added to PHSA with every clock cycle.
  FRQA := 1

  'Set APIN as an output.
  DIRA[StepPin]~~

  'Get the current System Counter value.
  Counter := CNT

  'Ramp up motor speed 
  REPEAT RampingSteps

    'Send out a high pulse on the step pin for the desired duration.
    PHSA := -HIGH_PULSE_WIDTH

    'Wait for a specified period of time before send sending another
    'high pulse to the step pin.
    WAITCNT(Counter += CycleOffset -= RampIncDec)

  'Run the motor at full speed until we get contrary input and then
  'stop this iteration on a completed revolution.
  REPEAT WHILE NOT INA[StopInputPin]
 
    'Send out a high pulse on the step pin for the desired duration.
    PHSA := -HIGH_PULSE_WIDTH

    'Wait for a specified period of time before send sending another
    'high pulse to the step pin.
    WAITCNT(Counter += CycleOffset)

    'Count the number of full speed steps.
    RunningSteps++

  'When the motor stops running, the total number of steps divided by
  'StepsPerRev must have a remainder of 0 to stop on a completed revolution.

  REPEAT UNTIL (TotalRampingSteps + RunningSteps) // StepsPerRev == 0
 
    'Send out a high pulse on the step pin for the desired duration.
    PHSA := -HIGH_PULSE_WIDTH

    'Wait for a specified period of time before send sending another
    'high pulse to the step pin.
    WAITCNT(Counter += CycleOffset)

    'Count the number of full speed steps.
    RunningSteps++

  'Ramp down the motor and come to a stop.
  REPEAT RampingSteps
 
    'Send out a high pulse on the step pin for the desired duration.
    PHSA := -HIGH_PULSE_WIDTH

    'Wait for a specified period of time before send sending another
    'high pulse to the step pin.
    WAITCNT(Counter += CycleOffset += RampIncDec)

PUB PulseTheStepPin(TotalSteps, StepPin, CycleOffset, MaxExeMotorSpeed, RampIncDec) | RampingSteps, RunningSteps, Counter

  'With this function, the total number of steps must be specified.
  'This function will ramp up a stepper motor until the maximum speed
  'is obtained and it will maintain maximum speed for a specified number
  'of steps, at which point it will begin a ramp down of the stepper motor
  'until it comes to a stop.

  'Determine the amount of ramping steps necessary before maximum speed
  'is obtained.
  RampingSteps := (CycleOffset - MaxExeMotorSpeed) / RampIncDec

  'If maximum speed can be obtained, determine the actual number of
  'ramping and running steps.
  IF TotalSteps > (RampingSteps * 2)
  
    RunningSteps := TotalSteps - (RampingSteps * 2)

   'Else ramp upto the half-way point and then ramp down.    
  ELSE
    RampingSteps := TotalSteps / 2
    RunningSteps := TotalSteps // 2    
  
  'Set up the CTRMODE of Counter A for NCO/PWM single-ended.
  CTRA[30..26] := %00100

  'Set the output pin for Counter A.
  CTRA[5..0] := StepPin

  'Set the value to be added to PHSA with every clock cycle.
  FRQA := 1

  'Set APIN as an output.
  DIRA[StepPin]~~

  'Get the current System Counter value.
  Counter := CNT

  'Ramp up the stepper motor to maximum speed.
  REPEAT RampingSteps
  
    'Send out a high pulse on the step pin for the desired duration.
    PHSA := -HIGH_PULSE_WIDTH

    'Wait for a specified period of time before send sending another
    'high pulse to the step pin.
    WAITCNT(Counter += CycleOffset -= RampIncDec)

  'Maintain maximum speed    
  REPEAT RunningSteps
   
    'Send out a high pulse on the step pin for the desired duration.
    PHSA := -HIGH_PULSE_WIDTH

    'Wait for a specified period of time before send sending another
    'high pulse to the step pin.
    WAITCNT(Counter += CycleOffset)

  'Ramp down the stepper motor and come to a stop. 
  REPEAT RampingSteps
  
    'Send out a high pulse on the step pin for the desired duration.
    PHSA := -HIGH_PULSE_WIDTH

    'Wait for a specified period of time before send sending another
    'high pulse to the step pin.
    WAITCNT(Counter += CycleOffset += RampIncDec) 
  